#!/usr/bin/env python
"""
xsandelay.py

Copyright (c) 2016 Apple, Inc.
All Rights Reserved.

This script delays Xsan startup on OS X El Capitan.
http://support.apple.com/kb/HT205706
"""

DELAY_SECONDS = 60  # adjust as needed

import os
import platform
import string
import struct
import subprocess
import sys
from syslog import syslog, LOG_NOTICE, LOG_ERR
from time import sleep, time

XSAN_LAUNCHD_PLIST = "/System/Library/LaunchDaemons/com.apple.xsan.plist"

def uptime():
    kbt = subprocess.check_output(["/usr/sbin/sysctl", "-b", "kern.boottime"])
    return time() - struct.unpack("qq", kbt)[0]

# do nothing if not running on OS X El Capitan
if string.join(platform.mac_ver()[0].split(".")[0:2], ".") != "10.11":
    syslog(LOG_NOTICE, "this script only runs on OS X El Capitan")
    sys.exit(0)

# do nothing if system uptime is greater than DELAY_SECONDS
if uptime() > DELAY_SECONDS:
    syslog(LOG_NOTICE, "this script only runs at startup")
    sys.exit(0)

# do nothing if Xsan volumes are already mounted
acfsMounts = subprocess.check_output(["/sbin/mount", "-t", "acfs"])
if len(string.split(acfsMounts, "\n")[0:-1]):
    syslog(LOG_NOTICE, "one or more Xsan volumes is already mounted")
    sys.exit(0)

wasStopped = False

# stop Xsan if found running within DELAY_SECONDS of boot time
while uptime() < DELAY_SECONDS:
    isXsanRunning = subprocess.call(["/bin/launchctl", "print", "system/com.apple.xsan"]) == 0
    if isXsanRunning and not wasStopped:
        syslog(LOG_NOTICE, "stopping Xsan")
        subprocess.call(["/bin/launchctl", "bootout", "system", XSAN_LAUNCHD_PLIST])
        wasStopped = True
    sleep(1)

# start Xsan if it was stopped
if wasStopped:
    syslog(LOG_NOTICE, "starting Xsan")
    subprocess.call(["/bin/launchctl", "bootstrap", "system", XSAN_LAUNCHD_PLIST])
